// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of Exception
//
// Exception is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// Exception is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include "Exception.h"

#include <gtest/gtest.h>

#include <sstream>
#include <string>

using namespace std;



// just make sure we can throw it
TEST(Exception, throw) {

    ASSERT_THROW( throw ( Exception( "e1" ) ), Exception );

}

// construct 'em

TEST(Exception, constructors) {

    Exception e("foo");
    ASSERT_STREQ( "foo", e.what() );

    Exception e1( e );
    ASSERT_STREQ( "foo", e1.what() );

    Exception e2( string( "bar" ) );
    ASSERT_STREQ( "bar", e2.what() );

}

// stack 'em
TEST(Exception, stack) {

    Exception e("foo");

    ASSERT_STREQ( "foo", e.what() );
    ASSERT_STREQ( "foo", e.begin()->c_str() );
    ASSERT_STREQ( "foo", (e.end() - 1)->c_str() );

    e.set_message( "bar" );
    ASSERT_STREQ( "barfoo", e.what() );
    ASSERT_STREQ( "bar", e.begin()->c_str() );
    ASSERT_STREQ( "foo", (e.end()-1)->c_str() );
}


// special interfaces
TEST(Exception, rethrow) {

    // no sugar
    Exception e( "foo" );
    e.set_rethrow_message( "bar" , 42 );

    ASSERT_NE( string::npos, string(e.what()).find( "file 'bar'" ) );
    ASSERT_NE( string::npos, string(e.what()).find( "line 42" ) );

    ASSERT_NE( string::npos, e.begin()->find( "file 'bar'" ) );
    ASSERT_NE( string::npos, e.begin()->find( "line 42" ) );

    ASSERT_STREQ( "foo", (e.end()-1)->c_str() );


    // add some sugar

    int line=__LINE__; const char* file = __FILE__; ASSERT_THROW( RETHROWME( e ), Exception );
    ostringstream sline;
    sline << "line " << line;

    ASSERT_NE( string::npos, string(e.what()).find( string( "file '" )  + file + "'" ) );
    ASSERT_NE( string::npos, string(e.what()).find( sline.str() ) );

}
