// --8<--8<--8<--8<--
//
// Copyright (C) 2013 Smithsonian Astrophysical Observatory
//
// This file is part of bp-save-history
//
// bp-save-history is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -->8-->8-->8-->8--

#include <algorithm>
#include <functional>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <tracefctxx/TraceFct.h>

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>
#include <bpipe/bpipe_axaf.h>

#include "param.h"
#include "field.h"

using namespace std;

void
process( Pars& pars) {
    TraceFct tf( "process" );
    BPipe *bpipe;
    BPipeOutput *bpo;

    void *data;

    size_t nphot;

    /* Setup bpipe input and output streams */

    if ( ( bpipe = bpipe_new() ) == NULL)
	tf.die( "couldn't open bpipe: %s",
		bpipe_strerror(bpipe_errno));

    if ( 0 != bpipe_input(bpipe, pars.input.c_str()) )
	tf.die( "couldn't open input file `%s': %s", pars.input.c_str(),
		 bpipe_strerror(bpipe_errno));

    if ( NULL == ( bpo = bpipe_output( bpipe, pars.output.c_str() ) ) )
	tf.die( "couldn't open output file `%s': %s", pars.output.c_str(),
		bpipe_strerror(bpipe_errno));

    list<Field> fields = gen_fields( bpipe, pars.field_names );

    if (NULL == (data = bpipe_map_alloc(bpipe,1, NULL)))
	tf.die( "unable to allocate memory for data packet: %s",
		bpipe_strerror(bpipe_errno));

    // calculate core positions for each field to be extended so that
    // they can dup the data
    for_each( fields.begin(),
	      fields.end(),
	      bind2nd( mem_fun_ref(&Field::set_data), data )
	      );


    if ( bpipe_write_hdr(bpipe) )
	tf.die( "bpipe_write_hdr: %s",bpipe_strerror(bpipe_errno));

    nphot = 0;
    bpipe_errno = BPNOERROR;


    while( bpipe_read_dpkts( bpipe, data, (size_t) 1 )   == 1) {

	for_each( fields.begin(),
		  fields.end(),
		  mem_fun_ref(&Field::copy_data)
		  );

	if(bpipe_write_dpkt( bpipe, data, bpo))
	    tf.die( "error writing photon %d", nphot);

    }

    if ( bpipe_errno != BPNOERROR )
	tf.die( "error reading photon %d: %s", nphot + 1,
		bpipe_strerror( bpipe_errno) );

    bpipe_delete( bpipe );
}

