// --8<--8<--8<--8<--
//
// Copyright (C) 2011 Smithsonian Astrophysical Observatory
//
// This file is part of bpmanip
//
// bpmanip is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -->8-->8-->8-->8--

#include "Fields.h"

#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <functional>

using std::vector;
using std::string;
using std::not1;


Fields::Fields( vector< string>& inc, vector< string >& exc ) {

    include.clear();
    exclude.clear();

    copy( inc.begin(), inc.end(), back_inserter(include) );
    copy( exc.begin(), exc.end(), back_inserter(exclude) );
}

vector<string>
Fields::allowed( vector < string>& input_fields ) {

    vector<string> allowed_fields;
    vector<string> final_fields;

    // if no include fields are specified, start with the full set
    if ( include.empty() )

	allowed_fields = input_fields;

    else

	remove_copy_if( input_fields.begin(), input_fields.end(),
			back_inserter(allowed_fields),
			not1(bind1st(WildmatV(), include))
			);

    if ( exclude.empty() )

	return allowed_fields;

    else
	remove_copy_if( allowed_fields.begin(), allowed_fields.end(),
			back_inserter(final_fields),
			bind1st(WildmatV(), exclude )
			);

    return final_fields;
}

vector<string>
Fields::rejected( vector < string>& input_fields ) {

    vector<string> deleted_fields;
    vector<string> final_fields;

    // if include fields are specified, insert non-matching
    // fields into the deleted list
    if ( ! include.empty() )

	remove_copy_if( input_fields.begin(), input_fields.end(),
			back_inserter(deleted_fields),
			bind1st(WildmatV(), include)
			);

    // if exclude fields are specified, insert matching
    // fields into the deleted list.  The matches are made
    // against the input fields list.  This may contain
    // fields implicitly deleted by the above logic, thus
    // leading to duplicates; these are removed later.
    // not the most efficient way to do things, but
    // the most straightforward.

    if ( ! exclude.empty() ) {

	remove_copy_if( input_fields.begin(), input_fields.end(),
			back_inserter(deleted_fields),
			not1(bind1st(WildmatV(), exclude ))
			);

	sort( deleted_fields.begin(), deleted_fields.end() );

	vector<string>::iterator end = unique( deleted_fields.begin(), deleted_fields.end() );
	deleted_fields.erase( end, deleted_fields.end() );

    }

    return deleted_fields;
}
