// --8<--8<--8<--8<--
//
// Copyright (C) 2011 Smithsonian Astrophysical Observatory
//
// This file is part of bpmanip
//
// bpmanip is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -->8-->8-->8-->8--

#include <string>
#include <vector>
#include <algorithm>
#include <iostream>
#include <tclap/CmdLine.h>

using std::string;
using std::vector;

#include <suplibxx/str.h>
#include <tracefctxx/TraceFct.h>

#include "Options.h"

#include "Fields.h"

#include "config.h"


Options::Options( int argc, char *argv[] ) {

    TraceFct tf( "Options" );

    try {

	TCLAP::CmdLine cmd("manipulate a bpipe stream", ' ', VERSION);

	TCLAP::ValueArg<std::string> inputArg( "i", "input", "input filename", false, "stdin", "string", cmd );

	TCLAP::ValueArg<std::string> outputArg( "o", "output", "output filename", false, "stdout", "string", cmd );

	TCLAP::ValueArg<std::string> includeArg( "", "include", "comma separated list of fields to include",
						 false, "", "string", cmd );

	TCLAP::ValueArg<std::string> excludeArg( "", "exclude", "comma separated list of fields to exclude",
						 false, "", "string", cmd );

	cmd.parse( argc, argv );

	input = inputArg.getValue();
	output = outputArg.getValue();

	vector< string > include;
	vector< string > exclude;

	suplib::tok( include, includeArg.getValue(), "," );
	suplib::tok( exclude, excludeArg.getValue(), "," );

	fields = Fields( include, exclude );

    }

    catch(TCLAP::ArgException &e ) {

	tf.die( "error parsing command line options: %s for arg %s\n", e.error().c_str(), e.argId().c_str() );

    }

}
