module( ..., package.seeall )

local raygen = require('saotrace.raygen')
local source = require( 'chandra.saotrace.raygen.source' )
local va = require('validate.args')
local validate = va.validate

local srcdir = os.getenv('srcdir') or '.'
local picket_db = table.concat( { srcdir, 'data', 'picket.rdb'}, '/' )

local package = package

require('deepcompare')

function setup()

   _G.setup()

   package.loaded['chandra.saotrace.raygen.source'] = nil
   source = require( 'chandra.saotrace.raygen.source' )

end

function test_point ()


   source.point{ name = 'point',
		 spectrum = { 33, 44,
				{ type = 'picket', file = picket_db },
			     },
	      }

   stash = raygen.intrinsic.stash

   -- start the source
   assert_true( deepcompare( { { name = 'point'},
			       func = 'start',
			       mod = 'source' 
			    },
			     stash[1]), 'start source' )

   -- the order of the following is dependent upon what happens in
   -- saotrace.raygen.source.create

   assert_equal( 'extent', stash[2].mod );
   assert_equal( 'point',    stash[2].func );

   local st = stash[3]
   assert_equal( 'spectrum', st.mod );
   assert_equal( 'mono',     st.func );
   assert_equal( 33,         st[1].energy );
   assert_equal( 1,          st[1].flux );

   local st = stash[4]
   assert_equal( 'spectrum', st.mod );
   assert_equal( 'mono',     st.func );
   assert_equal( 44,         st[1].energy );
   assert_equal( 1,          st[1].flux );

   local st = stash[5]
   assert_equal( 'spectrum', st.mod );
   assert_equal( 'mono',     st.func );
   assert_equal( 1,          st[1].energy );
   assert_equal( 1,          st[1].flux );

   local st = stash[6]
   assert_equal( 'spectrum', st.mod );
   assert_equal( 'mono',     st.func );
   assert_equal( 2,          st[1].energy );
   assert_equal( 2,          st[1].flux );

   -- finish the source
   assert_true( deepcompare( { { name = 'point' },
			       func = 'finish',
			       mod = 'source'
			    },
			     stash[#stash]), 'finish source' )

end

function test_multiple_autonamed()

   stash = raygen.intrinsic.stash

   source.point{ spectrum = 1.1 }
   source.point{ spectrum = 2.2 }
   source.point{ spectrum = 3.3 }

   local st = stash[1]

   assert_equal( 'source', st.mod );
   assert_equal( 'start',  st.func );


   assert_equal( 'point1',      stash[1][1].name );
   assert_equal( 'point1',      stash[2][1].name );
   assert_equal( 'point1:spectrum[1]',      stash[3][1].name );
   assert_equal( 'point1',      stash[4][1].name );

   assert_equal( 'point2',      stash[5][1].name );
   assert_equal( 'point2',      stash[6][1].name );
   assert_equal( 'point2:spectrum[1]',      stash[7][1].name );
   assert_equal( 'point2',      stash[8][1].name );


end
