#include <string>

using namespace std;

#include <gtest/gtest.h>

#include <cxcparamx/simple.h>

#include "argv.h"

using namespace cxcparamx;

const double dvalue = 3.2;
const double fvalue = 3.2f;
const int    ivalue = 4;
const string svalue = "stdin";

TEST(Open,BadFileArgV) {

    string filename( "does_not_exist" );

    Argv argv( "does_not_exist" );

    ASSERT_THROW( simple( argv.size(), argv() ), Exception );

}

TEST(Open,BadFile0) {

    string filename( "does_not_exist" );

    Argv argv( "does_not_exist" );

    ASSERT_THROW( simple( "Frankly my dear", argv.size(), argv() ), Exception );

}

TEST(Open,GoodFile) {

    Argv argv( "test" );

    cxcparamx::simple* pf = NULL;
    ASSERT_NO_THROW( pf = new simple( argv.size(), argv(), "r" ) );
    delete pf;
}

class NoArgs : public ::testing::Test {

protected:

    static void SetUpTestCase() {
	Argv argv( "test" );
	pf = new simple( argv.size(), argv(), "r" );
    }

    static void TearDownTestCase() {
	delete pf;
    }

    static cxcparamx::simple* pf;

};

cxcparamx::simple* NoArgs::pf = NULL;


TEST_F(NoArgs, String) {

    string value;

    ASSERT_NO_THROW( pf->get( "string", value ) );

    EXPECT_EQ( svalue, value );

    value.clear();

    ASSERT_NO_THROW( value = pf->getstr( "string" ) );

    EXPECT_EQ( svalue, value );
}

TEST_F(NoArgs, Double) {

    double value;

    ASSERT_NO_THROW( pf->get( "real", value ) );

    EXPECT_DOUBLE_EQ( dvalue, value );

    value = 0;

    ASSERT_NO_THROW( value = pf->getd( "real" ) );

    EXPECT_DOUBLE_EQ( dvalue, value );
}

TEST_F(NoArgs, Float) {

    float value;

    ASSERT_NO_THROW( pf->get( "real", value ) );

    EXPECT_FLOAT_EQ( fvalue, value );

    value = 0;

    ASSERT_NO_THROW( value = pf->getf( "real" ) );

    EXPECT_FLOAT_EQ( fvalue, value );
}

TEST_F(NoArgs, Short) {

    short value;

    ASSERT_NO_THROW( pf->get( "integer", value ) );

    EXPECT_EQ( ivalue, value );

    value = 0;

    ASSERT_NO_THROW( value = pf->gets( "integer" ) );

    EXPECT_EQ( ivalue, value );
}

TEST_F(NoArgs, Integer) {

    int value;

    ASSERT_NO_THROW( pf->get( "integer", value ) );

    EXPECT_EQ( ivalue, value );

    value = 0;

    ASSERT_NO_THROW( value = pf->geti( "integer" ) );

    EXPECT_EQ( ivalue, value );
}

TEST_F(NoArgs, Long) {

    long value;

    ASSERT_NO_THROW( pf->get( "integer", value ) );

    EXPECT_EQ( ivalue, value );

    value = 0;

    ASSERT_NO_THROW( value = pf->getl( "integer" ) );

    EXPECT_EQ( ivalue, value );
}

TEST_F(NoArgs, BoolTrue) {

    bool value;

    ASSERT_NO_THROW( pf->get( "true", value ) );

    EXPECT_TRUE( value );

    value = false;

    ASSERT_NO_THROW( value = pf->getb( "true" ) );

    EXPECT_TRUE( value );
}

TEST_F(NoArgs, BoolFalse) {

    bool value;

    ASSERT_NO_THROW( pf->get( "false", value ) );

    EXPECT_FALSE( value );

    value = true;

    ASSERT_NO_THROW( value = pf->getb( "false" ) );

    EXPECT_FALSE( value );
}
