/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2012 Smithsonian Astrophysical Observatory
 *
 * This file is part of imageblur
 *
 * imageblur is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * -->8-->8-->8-->8-- */

// -*-C++-*-
#ifndef CDF_H
#define CDF_H

#include <iostream>
#include <vector>
#include <iterator>
#include <string>

#include <limits>


class CDF {

public:
    CDF( std::string filename,
	 double xoffset = 0, double yoffset = 0, double scale = 1.0,
	 double pmin = 0, double pmax = 1, std::string clip = "cdf" ) ;
    CDF( ) {};

    struct Pixel {
	long int x;
	long int y;
	double p;

	bool operator< ( const Pixel& p2 ) const { return p < p2.p ; }
	bool operator< ( double prob ) const { return p < prob ; }

	friend bool operator< ( double prob, const Pixel& p ) { return prob < p.p ; }


	friend std::ostream& operator<< ( std::ostream& ostr,
					  const Pixel& p
					  ) {
	    ostr
		<< "x = " << p.x
		<< "\ty = " << p.y
		<< "\tp = " << p.p
		;

	    return ostr;
	}

	struct partial_sum {

	    double sum;
	    bool first;

	    partial_sum( ) : sum(0), first(true) {} ;

	    void operator() ( Pixel& pix ) {

		if ( first )
		    first = false;
		else
		    pix.p += sum;

		sum = pix.p;
	    }

	    operator double() { return sum ; }

	};

	struct sum {

	    double value;

	    sum( ) : value(0) {} ;

	    void operator() ( Pixel& pix ) { value += pix.p;  }
	    operator double() { return value ; }
	};

	struct divides {

	    double denom;

	    divides( double denom ) : denom(denom) {} ;

	    void operator() ( Pixel& pix ) { pix.p /= denom;  }
	};

	struct subtracts {

	    double addend;

	    subtracts( double addend ) : addend(addend) {} ;

	    void operator() ( Pixel& pix ) { pix.p -= addend;  }
	};

	struct clip {

	    double min, max;

	    clip( double min, double max ) : min(min), max(max) {};

	    bool operator() ( Pixel& pix ) {

		return pix.p <= min || pix.p > max ;

	    }


	};

    };

    typedef std::vector<CDF::Pixel> Pixels ;

    void dump( long max = 0);
    void dump( Pixels::iterator begin, Pixels::iterator end);
    void dump( Pixels::iterator begin, long end);

    Pixels pixels;

    double xoffset, yoffset, scale;


    void read_image( std::string filename );

    void find_pixel( const double prob, long int& x, long int& y);

    void offset( double& x, double &y );

};


#endif // ! defined CDF_H
