// --8<--8<--8<--8<--
//
// Copyright (C) 2012 Smithsonian Astrophysical Observatory
//
// This file is part of imageblur
//
// imageblur is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -->8-->8-->8-->8--

#include <tracefctxx/TraceFct.h>
#include <randlib_c/randlib.h>

#include "param.h"
#include "cdf.h"
#include "process.h"
#include "usage.h"
#include "config.h"

int main( int argc, char* argv[] ) {

    TraceFct tf( argv[0], 0, -1 );

    Params pars( argc, argv);

    try {

	if ( pars.help ) {
	    usage();
	    return 0;
	}

	else if (pars.version) {
	    cout << argv[0] << ' ' << VERSION << "\n";
	    return 0;
	}

	// initialize random number generator
	setall(pars.seed1, pars.seed2);
	for ( long n = 0 ; n < pars.block ; n++ )
	    initgn(1);

	// get distribution
	CDF cdf( pars.pdf,
		 pars.xoffset, pars.yoffset, pars.scale,
		 pars.pmin, pars.pmax, pars.clip );

	process( pars, cdf );
    }
    catch ( Exception &e ) {

	tf.die( e );

    }

    return 0;
}
