/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2013 Smithsonian Astrophysical Observatory
 *
 * This file is part of imageblur
 *
 * src is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * src is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 *       Free Software Foundation, Inc.
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <limits>
#include <string>

using namespace std;

#include <cxcparamx/simple.h>

#include <Exception/Exception.h>

#include "param.h"

Params::Params( int argc, char** argv ) {

    cxcparamx::simple pf( argc, argv );

    pf.get( "version", version );

    pf.get( "help", help );

    if ( version || help )
	return;

    pf.get( "input",  input );
    pf.get( "output", output );
    pf.get( "pdf",    pdf );
    pf.get( "pmin",   pmin );
    pf.get( "pmax",   pmax );
    pf.get( "clip",   clip );

    string offset;

    pf.get( "x0", offset );
    if ( offset == "center" )
	xoffset = numeric_limits<double>::max();
    else
	pf.get( "x0", xoffset );

    pf.get( "y0", offset );
    if ( offset == "center" )
	yoffset = numeric_limits<double>::max();
    else
	pf.get( "y0", xoffset );

    pf.get( "scale", scale );
    pf.get( "seed1", seed1 );
    pf.get( "seed2", seed2 );
    pf.get( "block", block );

    if (seed1 < 1 || seed1 > 2147483562)
	throw( Exception( "seed1 must have range [1,2147483562]") );

    if (seed2 < 1 || seed2 > 2147483398)
	throw( Exception( "seed2 must have range [1,2147483398]") );

    if (block < 0 || block > 1048575)
	throw( Exception( "block must have range [0,1048575]") );

}
