// --8<--8<--8<--8<--
//
// Copyright (C) 2012 Smithsonian Astrophysical Observatory
//
// This file is part of imageblur
//
// imageblur is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -->8-->8-->8-->8--


#include <tracefctxx/TraceFct.h>

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>
#include <bpipe/bpipe_axaf.h>

#include "process.h"
#include "cdf.h"

void
process( Params& pars, CDF& cdf ) {

    TraceFct tf( "process" );

    bpipe_errno = BPNOERROR;

    BPipe *bpipe;
    BPipeOutput *bpo;
    void  *photon;

    DVector3 *pos;

    // open up the input photon stream

    if ( NULL == ( bpipe = bpipe_new( ) ) )
	tf.die( "error creating bpipe: %s", bpipe_strerror( bpipe_errno ) );

    if ( bpipe_input( bpipe, pars.input.c_str() ) )
	tf.die( "error opening bpipe input stream `%s': %s", pars.input.c_str(), bpipe_strerror( bpipe_errno ) );

    if ( NULL == ( bpo = bpipe_output( bpipe, pars.output.c_str() ) ) )
	tf.die( "error opening bpipe output stream `%s': %s", pars.output.c_str(), bpipe_strerror( bpipe_errno ) );

    if ( NULL == ( photon = bpipe_map_alloc( bpipe, 1, NULL ) ) )
	tf.die( "error mapping bpipe: %s", bpipe_strerror( bpipe_errno ) );

    // we only care about the position field in the data packets
    if ( NULL ==
	 ( pos = (DVector3 *)bpipe_dpktf_datap( bpipe, photon, BP_AXAF_phot_pos, BPDType_DVector3 ) ) )
	tf.die( "error getting info for " BP_AXAF_phot_pos ": %s", bpipe_strerror( bpipe_errno ) );

    if ( bpipe_write_hdr( bpipe ) )
	tf.die( "error writing bpipe header: %s", bpipe_strerror( bpipe_errno ) );

    long nphot;
    for ( nphot = 1; bpipe_read_dpkts( bpipe, photon, 1 ) ; nphot++)
    {
	cdf.offset( pos->x, pos->y );

	if ( bpipe_write_dpkt( bpipe, photon, bpo ) )
	    tf.die( "error writing photon %d", nphot );
    }

    if ( bpipe_errno != BPNOERROR )
	tf.die( "error while reading photon %d", nphot );

    bpipe_delete( bpipe );
}
