// --8<--8<--8<--8<--
//
// Copyright (C) 2012 Smithsonian Astrophysical Observatory
//
// This file is part of imageblur
//
// imageblur is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -->8-->8-->8-->8--

#include <iostream>
#include <algorithm>
#include <cmath>

#include <fitsio.h>

#include <randlib_c/randlib.h>
#include <fits_exception.h>

#include "images.h"

using namespace std;

void
get_image_info( string filename, long* extent ) {

    int bitpix;
    int naxis;

    fitsfile *fptr;
    int status = 0;

    if ( fits_open_image( &fptr, filename.c_str(), READONLY, &status ) )
	throw( fits_exception("", filename ) );

    if ( fits_get_img_param( fptr, 2, &bitpix, &naxis, extent, &status ) )
	throw( fits_exception("", filename) );

    fits_close_file( fptr, &status );

}

void
write_image( string filename, vector<double>& image, long* extent ) {

    int status = 0;
    fitsfile *fptr;
    long npix = extent[0] * extent[1];

    if ( fits_create_file( &fptr, ('!' + filename).c_str(), &status ) )
	throw( fits_exception( "file creation", filename ) );

    if ( fits_create_img( fptr, -32, 2, extent, &status ) )
	throw( fits_exception( "image creation", filename ) );

    long fpixel[2] = { 1, 1 };
    if ( fits_write_pix( fptr, TDOUBLE, fpixel, npix, &image[0], &status ) )
	throw( fits_exception( "image write", filename ) );

    if ( fits_close_file( fptr, &status ) )
	throw( fits_exception( "file close", filename ) );

}

vector<double>&
read_image( string filename ) {

    int status = 0;
    fitsfile *fptr;

    if ( fits_open_image( &fptr, filename.c_str(), READONLY, &status ) )
	throw( fits_exception( "file open", filename ) );

    long extent[2];
    get_image_info( filename, extent );

    long npix = extent[0] * extent[1];

    long fpixel[2] = { 1, 1 };
    vector<double>* image = new vector<double>(npix);

    if ( fits_read_pix( fptr, TDOUBLE, fpixel, npix, NULL, &((*image)[0]), NULL, &status ) )
	throw( fits_exception( "image read", filename ) );

    if ( fits_close_file( fptr, &status ) )
	throw( fits_exception( "file close", filename ) );

    return *image;
}

void
normalize_image( vector<double>& image, double denom ) {

    if ( denom == 0 )
	denom = *( max_element( image.begin(), image.end()) );

    transform( image.begin(), image.end(), image.begin(), bind2nd(divides<double>(), denom ) );

}

