require 'lunatest'
require 'deepcompare'

raygen = require('saotrace.raygen' )

vobj = require('va').vobj

vspec = require( 'saotrace.raygen.jitter' ).vspec

function test_lj ()

   local ok, args = vobj:validate( vspec.lissajous,
				   { name = 'lj0' }
			      )
   assert_false( ok, args );
   assert_match( ': required but not specified', args );


   local ok, args = vobj:validate( vspec.lissajous,
			      { name = 'lj0',  pitch = 3 }
			      )
   assert_false( ok, args );
   assert_match( 'pitch: incorrect type', args );

   local ok, args = vobj:validate( vspec.lissajous,
			      { name = 'lj0',
				pitch = { period = 3, amp = 2} }
			      )
   assert_false( ok, args );
   assert_match( 'yaw: required', args );

   local ok, args = vobj:validate( vspec.lissajous,
			      { name = 'lj0',
				pitch = { period = 3, amp = 2}, yaw = 3 }
			      )
   assert_false( ok, args );
   assert_match( 'yaw: incorrect type', args );

   local ok, args = vobj:validate( vspec.lissajous,
				   { name = 'lj0',
				     pitch = { period = 3, amp = { 3, 'radian' }, phase = 3},
				     yaw   = { period = 3, amp = { 3, 'radian' }, phase = 3}
				  }
			      )
   assert_true( ok, args );

   assert_equal( 'lj0', args.name );
   for _, v in ipairs( { 'pitch', 'yaw' } ) do
     for _, v1 in pairs( { 'phase', 'period', 'amp' } ) do
        assert_equal( 3, args[v][v1][1], v .. '.' .. v1 )
     end
   end


   local ok, args = vobj:validate( vspec.lissajous,
				   { name = 'lj0',
				     pitch = { phase = 3, amp = { 4, 'radian' }, period = 5},
				     yaw   = { phase = 6, amp = { 7, 'radian' }, period = 8 }
				  }
			      )
   assert_true( ok, args );

   assert_equal( 3, args.pitch.phase[1] );
   assert_equal( 4, args.pitch.amp[1] );
   assert_equal( 5, args.pitch.period[1] );
   assert_equal( 6, args.yaw.phase[1] );
   assert_equal( 7, args.yaw.amp[1] );
   assert_equal( 8, args.yaw.period[1] );

end

function test_table ()

   local ok, args = vobj:validate( vspec.file,
				   {
				      name = 'f0',
				      file = 'foo.fits',
				      quat_col = 'q_attr',
				      optical_axis = {
					 { angle = 1, axis = { 0, 0, 1 } },
					 { angle = 2, axis = { 0, 1, 0 } },
					 { angle = 3, axis = { 1, 1, 1 } },
				      },

				      input_origin = {
					 { angle = 3, axis = { 1, 1, 1 } },
					 { angle = 2, axis = { 0, 1, 0 } },
					 { angle = 1, axis = { 1, 0, 0 } },
				      },
				   }
				)

   assert_true( ok, args, args );

   local axis = args.optical_axis
   assert_equal( 1, axis[1].angle[1], "optical_axis[1].angle" );
   assert_equal( 2, axis[2].angle[1], "optical_axis[2].angle" );
   assert_equal( 3, axis[3].angle[1], "optical_axis[3].angle" );
   assert_true( deepcompare( { 0, 0, 1}, axis[1].axis), "optical_axis[1].axis" );
   assert_true( deepcompare( { 0, 1, 0}, axis[2].axis), "optical_axis[2].axis" );
   assert_true( deepcompare( { 1, 1, 1}, axis[3].axis), "optical_axis[3].axis" );

   local axis = args.input_origin
   assert_equal( 3, axis[1].angle[1], "input_origin[1].angle" );
   assert_equal( 2, axis[2].angle[1], "input_origin[2].angle" );
   assert_equal( 1, axis[3].angle[1], "input_origin[3].angle" );
   assert_true( deepcompare( { 1, 1, 1}, axis[1].axis), "input_origin[1].axis" );
   assert_true( deepcompare( { 0, 1, 0}, axis[2].axis), "input_origin[2].axis" );
   assert_true( deepcompare( { 1, 0, 0}, axis[3].axis), "input_origin[3].axis" );

end

function test_table_components ()

   local ok, args = vobj:validate( vspec.file,
				 {
				    name = 'f0',
				    file = 'foo.fits',
				    axes = {
				       { col = 'ra',   axis = { 0,  0, 1 } },
				       { col = 'dec',  axis = { 0, -1, 0 } },
				       { col = 'roll', axis = { 1,  0, 0 } },
				    }

			         }
				)


   assert_true( ok, args, args );

   local axis = args.axes
   assert_equal( 'ra',   axis[1].col, "axis[1].col" );
   assert_equal( 'dec',  axis[2].col, "axis[2].col" );
   assert_equal( 'roll', axis[3].col, "axis[3].col" );
   assert_true( deepcompare( { 0,  0, 1}, axis[1].axis), "axis[1].axis" );
   assert_true( deepcompare( { 0, -1, 0}, axis[2].axis), "axis[2].axis" );
   assert_true( deepcompare( { 1,  0, 0}, axis[3].axis), "axis[3].axis" );


   local ok, args = vobj:validate( vspec.file,
				 {
				    name = 'f0',
				    file = 'foo.fits',
				    axes = {
				       { col = 'ra',   axis = { 0,  0, 1 } },
				       { col = 'dec',  axis = { 0, -1, 0 } },
				    }

			         }
				)


   assert_false( ok, args, args );


end


lunatest.run()
