local _M = {}

local table = require('table')
local top_srcdir = require('os').getenv('top_srcdir') or '.'

local db = table.concat( { top_srcdir, 'tests', 'data', 'test.rdb'}, '/' )

local rdb = require( 'rdb' )


function _M.test_table( )

   local nmatch = rdb.select{ file = db,
			      match = { x = 3 } }

   assert_equal( 1, #nmatch )

   assert_equal( 3, nmatch[1].x )
   assert_equal( 2, nmatch[1].y )

end

function _M.test_func( )

   local nmatch = rdb.select{ file = db,
			    match = function(rec)
				       return rec.y < 9
				    end,
			   }


   assert_equal( 2, #nmatch )

   assert_equal( 3, nmatch[1].x )
   assert_equal( 2, nmatch[1].y )

   assert_equal( 5, nmatch[2].x )
   assert_equal( 4, nmatch[2].y )

end

function _M.test_iterator( )

   local data = rdb.slurp( db )

   local iter = rdb.select{ file = db,
			    match = function(rec)
				       return rec.y < 9
				    end,
			    mode = 'iterator'
			   }

   local nrow = 1
   for rec, idx in iter do

      assert_equal( nrow, idx )
      assert_equal( data[idx].x, rec.x )
      assert_equal( data[idx].y, rec.y )

      nrow = nrow + 1
   end

   assert( 3, nrow )


end

function _M.test_match_table( )

   local nmatch = rdb.select{ file = db,
			      match = { y = { 2, 9 } }
			   }

   assert_equal( 2, #nmatch )

   assert_equal( 3, nmatch[1].x )
   assert_equal( 2, nmatch[1].y )

   assert_equal( 22, nmatch[2].x )
   assert_equal( 9, nmatch[2].y )

end

return _M
