module( ..., package.seeall )

local math = math
local sqrt = math.sqrt
local deg2rad = math.pi / 180
local rad2deg = 1 / deg2rad

local coords = require( 'saotrace.raygen.coords' )

function test_delta_equatorial( )

   local tests = {

      { ra_s = 0, dec_s = 0,
	ra_f = 10, dec_f = 0,
	theta = 10, pa = 90
     },

      { ra_s = 0, dec_s = 0,
	ra_f = 0, dec_f = 10,
	theta = 10, pa = 0
     },

      { ra_s = 0, dec_s = 0,
	ra_f = -10, dec_f = 0,
	theta = 10, pa = -90
     },

      { ra_s = 0, dec_s = 0,
	ra_f = 0, dec_f = -10,
	theta = 10, pa = 180
     },

      { ra_s = 1, dec_s = 0,
	ra_f = -1, dec_f = 0,
	theta = 2, pa = -90,
     },

      { ra_s = 1, dec_s = 1,
	ra_f = -1, dec_f = -1,
	theta = sqrt(8), pa = -135,
	dtheta = 1e-4, dphi = .01
     },

   }

   for idx,v in ipairs( tests ) do

      local inp = {}

      for k, a in pairs( v ) do
	 inp[k] = a * deg2rad
      end


      local theta, pa = coords.delta_equatorial( inp.ra_s, inp.dec_s,
						 inp.ra_f, inp.dec_f )

      theta = theta * rad2deg
      pa = pa * rad2deg
      assert_equal( v.theta, theta, v.dtheta or 0, idx .. ': theta' )
      assert_equal( v.pa, pa, v.dphi or 0, idx .. ': pa' )

   end

end

