module( ..., package.seeall )

local table = require( 'table' )
local os = require( 'os' )

local deepcompare = require( 'deepcompare' )

local srcdir = os.getenv('srcdir') or '.'

local ea_db = table.concat( { srcdir, 'data', 'annulus.rdb'}, '/' )

local utils = require( 'saotrace.raygen.ea.utils.ring_n_spoke' )
local va = require('validate.args')
local validate = va.validate

function test_rings_single( )

   local pars = { rmin = 1,
		  rmax = 10,
		  nspokes = 1,
		  nrings = 1,
		  dist = 'area'
	       }

   local ring = 1
   local r
   for spot in utils.ringNspoke( pars ) do

      r = spot.r

      ring = ring + 1
   end

   assert_equal( 2, ring )

   amin = r^2 - pars.rmin^2;
   amax = pars.rmax^2 - r^2;

   assert_equal( amin, amax, "ring area" )

end


function test_rings_double( )

   local pars = { rmin = 1,
		  rmax = 10,
		  nspokes = 1,
		  nrings = 2,
		  dist = 'area'
	       }

   local exp = { pars.rmin, pars.rmax }

   local ring = 1
   for spot in utils.ringNspoke( pars ) do

      assert_equal( exp[ring], spot.r )

      ring = ring + 1
   end

   assert_equal( 3, ring )

end

function check_areas( pars )

   local ring = 1
   local area = {}
   local lr

   for spot in utils.ringNspoke( pars ) do

      if ring > 1 then
	 area[ring-1] = spot.r^2 - lr^2
      end

      if ring > 2 then
	 assert_equal( area[ring-1], area[ring-2] )
      end

      lr = spot.r

      ring = ring + 1
   end

   assert_equal( pars.nrings+1, ring )

   assert_equal( pars.rmax, lr )

end

function test_rings_odd( )

   local pars = { rmin = 1,
		  rmax = 10,
		  nspokes = 1,
		  nrings = 5,
		  dist = 'area'
	       }

   check_areas( pars )
end


function test_rings_even( )

   local pars = { rmin = 1,
		  rmax = 10,
		  nspokes = 1,
		  nrings = 4,
		  dist = 'area'
	       }

   check_areas( pars )

end


