#include <tparam/tparam.h>
#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "main.h"
#include "common.h"
#include "image_write.h"

/*************************************************************************/

/* internal routines, needed to fill method tables below */

static void gparam_binary(paramfile pf);
static int setup_hdr(FILE *stream, PixStorage pix_storage,
		     long i_min, long i_max, long j_min, long j_max);
static int setup_raw(FILE *stream, PixStorage pix_storage,
		     long i_min, long i_max, long j_min, long j_max);
static void binary_hdr(BackEndType type, void *hdr, size_t sizeof_hdr,
		       HdrMapEntry map[], size_t n_entries);
static void start_i(long i, size_t nj);
static void write_pix(CoreWeight *pix, long j_min, size_t npix);
static void write_zero(long j_min, size_t npix);
static void end_i(void);
static void cleanup(void);

/*************************************************************************/

/* method tables */

ImageDump
dump_binary_raw =
{
  "binary-raw",		/* the unique identifier for an image dump */
  gparam_binary,	/* parameter parser */
  setup_raw,		/* setup */
  NULL,
  NULL,
  NULL,
  binary_hdr,
  start_i,
  write_pix,
  write_zero,
  end_i,
  cleanup
};

ImageDump
dump_binary_hdr =
{
  "binary-hdr",		/* the unique identifier for an image dump */
  gparam_binary,	/* parameter parser */
  setup_hdr,		/* setup */
  NULL,
  NULL,
  NULL,
  binary_hdr,
  start_i,
  write_pix,
  write_zero,
  end_i,
  cleanup
};

/*************************************************************************/

/*
   file global storage for methods
 */

/* the dump stream, set by setup */
static FILE *fdump = NULL;

/* the current value of i. set by start_i */
static long current_i;

#define DUMPFUNC "binary dump"

/*************************************************************************/

static void
gparam_binary(paramfile pf)
{
  tf_enter(DUMPFUNC " gparam");

  tf_leave();
}

static int
setup_hdr(
  FILE	     *stream,
  PixStorage  pix_storage,
  long	      i_min,
  long	      i_max,
  long	      j_min,
  long	      j_max
)
{
  tf_enter(DUMPFUNC " setup_hdr");

  fdump = stream;

  image_write_setup();

  c.backend->write_header_raw(pix_storage, i_min, i_max, j_min, j_max,
			       &dump_binary_hdr);

  tf_leave();
  return 0;
}

static int
setup_raw(
  FILE	     *stream,
  PixStorage  pix_storage,
  long	      i_min,
  long	      i_max,
  long	      j_min,
  long	      j_max
)
{
  tf_enter(DUMPFUNC " setup_raw");

  fdump = stream;

  image_write_setup();

  tf_leave();
  return 0;
}

static void
binary_hdr(BackEndType type, void *hdr, size_t sizeof_hdr, HdrMapEntry map[],
	   size_t n_entries)
{
  tf_enter(DUMPFUNC " binary_hdr_entry");

  c.binary.hdr(fdump, type, hdr, sizeof_hdr, map, n_entries);

  tf_leave();
}

static void
start_i(long i, size_t nj)
{
  tf_enter(DUMPFUNC " start_i");

  current_i = i;

  if ( c.binary.start_i(fdump, nj) )
    tf_exit(ExitERR_fputs, "error at i=%d at start of i dump", i);

  tf_leave();
}

static void
write_pix(CoreWeight *pix, long j_min, size_t npix)
{
  tf_enter(DUMPFUNC " write_pix");

  if ( c.binary.write_pix(fdump, pix, npix) )
    tf_exit(ExitERR_fputs, "error at (i=%d, j=%d) writing %d pixels",
		    current_i, j_min, npix);

  tf_leave();
}

static void
write_zero(long j_min, size_t npix)
{
  tf_enter(DUMPFUNC " write_zero");

  if ( c.binary.write_zero(fdump, npix) )
    tf_exit(ExitERR_fputs, "error at (i=%d, j=%d) writing %d pixels",
		    current_i, j_min, npix);

  tf_leave();
}

static void
end_i(void)
{
  tf_enter(DUMPFUNC " end_i");

  if ( c.binary.end_i(fdump) )
    tf_exit(ExitERR_fputs, "error at i=%d at end of i dump",
		    current_i);

  tf_leave();
}

static void
cleanup(void)
{
  image_write_cleanup();
}
