#include <string.h>

#include <tparam/tparam.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>
#include <suplib/debug.h>
#include <suplib/str.h>

#include "main.h"
#include "common.h"

#include "image_write.h"
#include "gparam.h"

static TokListToken
filter_mode[] =
{
  {"bin",	FilterMode_BIN},
  {"mask",	FilterMode_MASK},
  {"negmask",	FilterMode_NEGMASK},
  {"none",	FilterMode_NONE},
  {"pass",	FilterMode_PASS},
  {"quantidx",	FilterMode_QUANTIZEIDX},
  {"quantize",	FilterMode_QUANTIZEPOS},  /* for backwards compatibility */
  {"quantpos",	FilterMode_QUANTIZEPOS},
};
static TokList filter_modes = GenTokList(filter_mode);

static TokListToken
normalize_type[] =
{
  {"bin",	Normalize_BIN},
  {"image",	Normalize_IMAGE},
};
static TokList normalize_types = GenTokList(normalize_type);

#define DELIMS " \t,"

void
gparam(int argc, char *argv[])
{
  paramfile pf;
  int i;
  char dummy[NAME_LENGTH+1];
  char *token;

  tf_enter("gparam");

  pf = tparamopen(NULL, argv, argc, "r");
  c.version = tpgetb(pf, "version");

  /* only get other stuff if version isn't set */
  if (! c.version)
  {
    /* get debug flags */
    tpgetstr(pf, "debug", dummy, NAME_LENGTH, 0);
    debug_init(dummy);

    tpgetstr(pf, "input", c.input, NAME_LENGTH, 1);
    tpgetstr(pf, "output", c.output, NAME_LENGTH, 1);

    tpgetstr(pf, "quant_method", dummy, NAME_LENGTH, 1);
    c.backend = NULL;

    for (i = 0 ; i < n_backends ; i++)
      if (!strcmp(dummy, backend[i]->name)) c.backend = backend[i];

    if (! c.backend)
      tf_exit(ExitERR_misc, "unknown quantization method: %s", dummy);
    c.backend->gparam(pf);

    c.dump_image = tpgetb(pf, "create_image");
    if (c.dump_image)
    {
      tpgetstr(pf, "image_file", c.image_file, NAME_LENGTH, 1);

      tpgetstr(pf, "image_fmt", dummy, NAME_LENGTH, 1);
      c.dump = NULL;
      for (i = 0 ; i < n_imagedumps ; i++)
	if (!strcmp(dummy, imagedump[i]->name)) c.dump = imagedump[i];

      if (! c.dump)
	tf_exit(ExitERR_misc, "unknown image format: %s", dummy);
      c.dump->gparam(pf);

      gparam_image_write(pf);

    }

    tpgetstr(pf, "filter_mode", dummy, NAME_LENGTH, 1);
    c.filter = tokmatch(dummy, &filter_modes);
    if (c.filter == -1)
	tf_exit(ExitERR_illegalArg, "unknown filter mode `%s'", dummy);


    if (c.filter == FilterMode_NONE && ! c.dump_image)
    {
      tf_message("filter_mode=`none' and create_image=`no':\n");
      tf_message("no output will be produced\n");
    }

    c.x_off = tpgetd(pf, "xcent");
    c.y_off = tpgetd(pf, "ycent");

    c.clip = tpgetb(pf, "clip");

    tpgetstr(pf, "norm_area", dummy, NAME_LENGTH, 0);
    i = str_tokcnt(dummy, DELIMS);
    for (i = 0, token = strtok(dummy, DELIMS) ;
	 token ;
	 token = strtok(NULL, DELIMS), i++ )
    {
      if ( tokmatch(token, &normalize_types) == -1)
	tf_exit(ExitERR_illegalArg,
			"illegal normalization flag: `%s'", token);
      c.norm |= tokmatch(token, &normalize_types);
    }

    c.join_pad = tpgeti(pf, "join_pad");
    c.extend_pad = tpgeti(pf, "extend_pad");

    c.bufsz = 1048576 * tpgetd(pf, "bufsz");
  }
  
  paramclose(pf);
  tf_leave();
}
