#ifndef MAIN_H
#define MAIN_H

#include <stddef.h>
#include <stdio.h>

#include <rbtree/rbtree.h>
#include <tparam/tparam.h>

#include "spatquant.h"
#include "rwbpipe.h"
#include "wedge.h"

#define NAME_LENGTH 255

/* the type of a pixel */
typedef double CoreWeight;

typedef enum
{
  FilterMode_BIN,
  FilterMode_PASS,
  FilterMode_QUANTIZEPOS,
  FilterMode_QUANTIZEIDX,
  FilterMode_MASK,
  FilterMode_NEGMASK,
  FilterMode_NONE
} FilterMode;

typedef enum
{
  Normalize_BIN		= 0x01,
  Normalize_IMAGE	= 0x02
} NormalizeWhat;

typedef struct
{
  long i;
  RBTree j;
} Inode;

typedef struct
{
  long j_min;
  long j_max;
  CoreWeight *wt;
} Jnode;

typedef struct
{
  double x, y;
  long i, j;
} CoordIndex;

typedef enum
{
  CORE,
  SPARSE
} PixStorage;

typedef struct
{
  size_t offset;
  size_t size;
} HdrMapEntry;

typedef struct
{
  char  *name;		/* the unique identifier for an image dump */
  void   (*gparam)(paramfile pf);
  int	 (*setup)(FILE *stream, PixStorage pix_storage, long i_min, long i_max,
				long j_min, long j_max);

  void   (*ascii_hdr_start)(size_t len);
  void   (*ascii_hdr_entry)(char *var, char *format, ...);
  void   (*ascii_hdr_end)(void);
  void   (*binary_hdr)(BackEndType type, void *hdr, size_t sizeof_hdr,
		       HdrMapEntry map[], size_t n_entries);
  void   (*start_i)(long i, size_t nj);
  void   (*write_pix)(CoreWeight *pix, long j_min, size_t npix);
  void   (*write_zero)(long j_min, size_t npix);
  void   (*end_i)(void);
  void   (*cleanup)(void);
}ImageDump;

typedef struct
{
  char  *name;		/* the unique identifier for a backend */
  BackEndType type;
  void  (*gparam)(paramfile pf);
  int	(*setup)(void);
  int	(*i_limits)(long *min_i, long *max_i, unsigned long *nbins);
  void	(*j_limits)(long i, long *min_j, long *max_j);
  int	(*quantize)(double x_in, double y_in, CoordIndex *ci);
  void	(*bin_pos)(long i, long j, CoordIndex *ci);
  double	(*bin_area)(long i, long j);
  void	(*j_image_limits)(long i, long j_min, long j_max, long *min_j,
			  long *max_j);
  int   use_image_limits;

  void	(*write_header_raw)   (PixStorage pix_storage, long i_min, long i_max,
				long j_min, long j_max, ImageDump *dump);
  void	(*write_header_ascii) (PixStorage pix_storage, long i_min, long i_max,
				long j_min, long j_max, ImageDump *dump);

  int	(*cleanup)(void);
} BackEnd;

typedef struct
{
  void (*hdr)(FILE *stream, BackEndType type, void *hdr, size_t sizeof_hdr,
	      HdrMapEntry map[], size_t n_entries);
  int  (* start_i)(FILE *stream, size_t n);
  int  (* end_i)(FILE *stream);
  int  (* write_pix)(FILE *stream, CoreWeight *data, size_t n);
  int  (* write_zero)(FILE *stream, size_t n);
} BinaryIO;

typedef struct
{
  char input[NAME_LENGTH+1];
  char output[NAME_LENGTH+1];

  int output_pixels;

  char image_file[NAME_LENGTH+1];
  int dump_image;	/* dump image out to file */

  FilterMode filter;
  int clip;

  int version;

  long join_pad;
  long extend_pad;

  size_t bufsz;

  double x_off;
  double y_off;

  ImageFormat image_fmt;

  NormalizeWhat norm;

  BackEnd *backend;
  ImageDump *dump;
  BinaryIO binary;
} Common;

#endif /* !MAIN_H */

