/*
 Action routines called by the redblack tree routines to normalize pixel
 values by the area of the pixel. 
*/

#include <stdlib.h>
#include <stdio.h>


#include <rbtree/rbtree.h>
#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "main.h"
#include "common.h"
#include "norm_actions.h"

static int j_norm(void *dp);

/* as the routines are called by the redblack routines with a
   very limited number of arguments,  file-global variables are used to pass
   extra information to the action functions
*/

static long i;			/* keep track of i index for j routines */

/*
   this routine is called by the redblack traversal routine.  it works
   on an i-node. normalizing the node's associated j-tree.
*/
int
i_norm(void *dp)
{
  Inode *inode = (Inode *) dp;

  i = inode->i;
  rbtree_traverse(inode->j, j_norm, LEFT_TO_RIGHT);
  return 0;
}

/*
   this action routine is invoked by the redblack traversal routine at
   the behest of i_norm. 
*/
static int
j_norm(void *dp)
{
  Jnode *jnode = (Jnode *) dp;
  CoreWeight *wt = jnode->wt;
  CoreWeight *wt_lim = jnode->wt + jnode->j_max - jnode->j_min + 1;
  long j;

  for (j = jnode->j_min ; wt < wt_lim ; j++, wt++)
    if (*wt) *wt /= c.backend->bin_area(i, j);
  return 0;
}
