#include <stdio.h>
#include <string.h>

#include <tracefct/exiterrvals.h>
#include <tracefct/tracefct.h>

#include "rwbpipe.h"
#include "main.h"
#include "common.h"

/* simple routine to get the data pointer for a named data packet field */
static char *dpktf_data( Bpipe *bpipe, char *name, DpktField **field ) {

  char* ptr = NULL;
  DpktField *tfield;

  tf_enter( "dpktf_data" );
  tfield = bpipe_dpktf( bpipe->bpipe, name );

  if ( NULL == tfield )
    tf_exit( ExitERR_misc, "Unable to find field %s", name );

  ptr = bpipe_dpktf_data( tfield, bpipe->data );
  if ( NULL == ptr )
    tf_exit( ExitERR_misc, bpipe_strerror( bpipe_errno ) );

  tf_leave( );
  if ( field ) *field = tfield;
  return ptr;
}


void init_bpipe( char* input, char* output, Bpipe* pipe,
		 BpipePhoton* photon ) {

  tf_enter( "init_bpipe" );

  if( NULL == (pipe->bpipe=bpipe_new( )) )
    tf_exit( ExitERR_misc, bpipe_strerror( bpipe_errno ) );

  if ( bpipe_input( pipe->bpipe, input ) )
    tf_exit( ExitERR_fopen, bpipe_strerror( bpipe_errno ) );

  if ( FilterMode_NONE != c.filter) {

    if ( NULL == (pipe->outstream=bpipe_output( pipe->bpipe, output ) ) )
      tf_exit( ExitERR_fopen, bpipe_strerror( bpipe_errno ) );

    if ( FilterMode_BIN == c.filter ) {

      /* loop thru the data packets and  delete all but position and wt */
      void*  last = NULL;     /* initialize it for the first time through */
      DpktField *dpktf=NULL;

      while ( (dpktf=bpipe_dpktf_next( pipe->bpipe, &last )) ) {

	const char* name = bpipe_dpktf_name( dpktf );

	if ( 0 != strcmp( name, BP_AXAF_phot_wt ) &&
	     0 != strcmp( name, BP_AXAF_phot_pos ) ) {

	  if ( 0 != bpipe_dpktf_delete( pipe->bpipe, name, BPDSite_OUTPUT,
					pipe->outstream ) )
	    tf_exit( ExitERR_misc, "Unable to delete `%s'", name );

	}

      }

    }

  }

  /* add a weight data packet, just in case it doesn't exist */
  bpipe_dpktf_add( pipe->bpipe, BP_AXAF_phot_wt, BPDType_double, NULL );

  /* map data packet fields */
  if ( NULL == (pipe->data = bpipe_map_alloc( pipe->bpipe, 1, NULL ))  )
    tf_exit( ExitERR_illegalArg, bpipe_strerror( bpipe_errno ) );

  if ( FilterMode_NONE != c.filter) {

    /* output header. must do this after calling bpipe_map */
    if ( bpipe_write_hdr( pipe->bpipe ) )
      tf_exit( ExitERR_illegalArg, bpipe_strerror( bpipe_errno ) );

  }
  
  photon->weight = (double*) dpktf_data( pipe, BP_AXAF_phot_wt, NULL );

  /* just in case we added this, set it to unity */
  *(photon->weight) = 1;

  /* the position may be a DVector3, DVector2, or an array. gotta
     deal with 'em all */
  
  {
    DpktField *dpktf;
    void *pos = (DVector3*) dpktf_data( pipe, BP_AXAF_phot_pos, &dpktf);

    switch( bpipe_dpktf_type( dpktf ) )
    {
    case BPDType_DVector3:
      {
	DVector3 *p = (DVector3 *) pos;
	photon->x = &p->x;
	photon->y = &p->y;
	photon->z = &p->z;
      }
      break;
      
    case BPDType_DVector2:
      {
	DVector2 *p = (DVector2 *) pos;
	photon->x = &p->x;
	photon->y = &p->y;
	photon->z = &photon->z_dummy;
      }
      break;

    case BPDType_double:
      {
	double *p = (double *) pos;
	BPMatrix *bpm = bpipe_dpktf_matrix( dpktf, BPDSite_CORE, NULL);
	/* only can deal with one dimensional data, 2 or three elements */
	if ( bpm->nd != 1 || bpm->ne != 2 || bpm->ne != 3)
	  tf_exit( ExitERR_misc,
		   "position dpkt field has more than one dimension or\n"
		   "too many or too few elements" );

	photon->x = p + 0;
	photon->y = p + 1;
	photon->z = bpm->ne == 3 ? p + 3 : &photon->z_dummy;
	bpipe_matrix_delete( bpm );
      }
      break;

    default:
      tf_exit( ExitERR_misc,
	       "unexpected bpipe data type for position: `%s'",
	       bpipe_datatype_name( bpipe_dpktf_type( dpktf ) ) );

    }

  }

  tf_leave();
  return;
}
