#include <stdio.h>
#include <limits.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>
#include <suplib/debug.h>

#include "main.h"
#include "common.h"
#include "image_actions.h"
#include "sparse_image_dump.h"

void
sparse_image_dump(RBTree Itree, long min_i, long max_i, FILE *fimage)
{
  long j_min = LONG_MAX;
  long j_max = LONG_MIN;
  long i_min, i_max;

  tf_enter("sparse_image_dump");

  /* get min and max pixel values */

  if (c.clip)
  {
    i_min = min_i;
    i_max = max_i;
  }
  else
  {
    i_min = LONG_MAX;
    i_max = LONG_MIN;

    if (rbtree_count(Itree))
    {
      i_min = ((Inode *) rbtree_min(Itree))->i;
      i_max = ((Inode *) rbtree_max(Itree))->i;

      if (c.backend->use_image_limits)
      {
	RBNode node = rbtree_min_node(Itree);
	do
	{
	  RBTree tree = ((Inode *) rbtree_node_get_data(node))->j;
	  Jnode *jnode;

	  jnode = (Jnode *) rbtree_min(tree);
	  if (jnode->j_min < j_min)
	    j_min = jnode->j_min;

	  jnode = (Jnode *) rbtree_max(tree);
	  if (jnode->j_max > j_max)
	    j_max = jnode->j_max;

	} while ((node = rbtree_next_node(Itree, node, LEFT_TO_RIGHT)));

	if (dbflag("pix_limits"))
	  tf_message("pix_limits: %ld %ld %ld %ld\n", i_min, i_max,
						 j_min, j_max);
	if (dbflag("coord_limits"))
	{
	  CoordIndex min, max;
	  c.backend->bin_pos(i_min, j_min, &min);
	  c.backend->bin_pos(i_max, j_max, &max);
	  tf_message("coord_limits: %f %f %f %f\n", min.x, min.y,
					       max.x, max.y);
	}

	if (dbflag("dimen"))
	  tf_message("dimen: %ld %ld\n", i_max - i_min + 1,
				    j_max - j_min + 1);
      }
    }
  }

  c.dump->setup(fimage, SPARSE, i_min, i_max, j_min, j_max);

  image_free_setup(i_min, i_max, j_min, j_max, fimage);
  rbtree_delete(Itree, i_image_free, IN_ORDER, LEFT_TO_RIGHT);
  image_free_cleanup();

  c.dump->cleanup();

  tf_leave();
}

