#ifndef WEDGE_H
#define WEDGE_H

#define NAME_LENGTH 255

typedef enum {
  WithinWedge,
  NotWithinWedge
} IfoundIt;
    

/* Start of the Annuli struct which will be used when rtype=discrete|file */
typedef struct {

  double theta_min              /* the minimum angle (degrees) of the wedge */;
  double theta_max               /*the maximum angle (degrees) of the wedge */;

} Wedge;

typedef struct {

  double r_min;
  double r_max;
  double r_min_sqr               /* The min radius (squared) of the annulus */;
  double r_max_sqr               /* The max radius (squared) of the annulus */;
  int num_wedges              /* An annulus has num_wedges number of wedges */;
  Wedge* wedge                        /* The pointer to the array of Wedges */;

} Annulus;

typedef struct {

  int      num_annulus       /* An annuli has num_annulus number of Annulus */;
  Annulus* annulus                   /* The pointer to the array of Annulus */;

} Annuli;
/* End of the Annuli struct which will be used when rtype=discrete|file */

/* To be used if rtype=exp */
typedef struct {

  double base;
  double constant;
  double lnbase;
  int    num_wedges;
  Wedge* wedge;

} AnnuliExp;

/* To be used if rtype=linear */
typedef struct {

  double deltaR                         /* deltaR = (n+1) DeltaR - n deltaR */;
  int    num_wedges;
  Wedge* wedge;

} AnnuliLinear;

int  cmp_annulus( const void* k, const void* datum );
int  cmp_Wedge_theta( const void* k, const void* datum );
void calculate_wedges( Wedge** wedge_ptr, char* tmin_degrees, int num_wedges );
void print_Annuli( FILE* fp, Annuli* ptr );
void print_AnnuliExp( FILE* fp, AnnuliExp* ptr );
void print_AnnuliLinear( FILE* fp, AnnuliLinear* ptr );
AnnuliExp* read_wedge_exp( char str[] );
AnnuliLinear* read_wedge_linear( char str[] );
Annuli* read_wedge_rdb_file( char filename[] );
Annuli* read_wedge_list( char str[] );
int str2d( char* txt, double* result );
int str2i( char* txt, int* result );

#endif
