#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <tracefct/exiterrvals.h>
#include <suplib/str.h>
#include <tracefct/tracefct.h>

#include "wedge.h"

AnnuliExp* read_wedge_exp( char str[] ) {

  AnnuliExp* annuli=NULL;
  char** annuli_exp_tokens=NULL;
  int num_tokens=0;

  tf_enter( "read_wedge_exp" );

  annuli = (AnnuliExp*) malloc( sizeof( AnnuliExp ) );
  if ( NULL == annuli )
    tf_exit( ExitERR_alloc, "Unable to allocate memory for annuli" );

  num_tokens = str_tokenize( str, ",", &annuli_exp_tokens, 1, 1 );
  if ( 4 != num_tokens )
    tf_exit( ExitERR_illegalArg, "Illegal entry `%s', expected 4 tokens",
	     str );

  if ( EXIT_FAILURE == str2d( annuli_exp_tokens[ 0 ], &(annuli->constant) ) )
    tf_exit( ExitERR_illegalArg, "Illegal entry `%s' for the constant",
	     annuli_exp_tokens[ 0 ] );

  if ( EXIT_FAILURE == str2d( annuli_exp_tokens[ 1 ], &(annuli->base) ) )
    tf_exit( ExitERR_illegalArg, "Illegal entry `%s' for the base",
	     annuli_exp_tokens[ 1 ] );

  if ( annuli->base < 0 )
    tf_exit( ExitERR_illegalArg, "Illegal entry `%s' for the base, "
	     "it must be a positive number", annuli_exp_tokens[ 1 ] );

  annuli->lnbase = log( annuli->base );

  if ( EXIT_FAILURE == str2i( annuli_exp_tokens[ 2 ], &(annuli->num_wedges) ) )
    tf_exit( ExitERR_illegalArg, "Illegal entry `%s' for the number of wedges",
	     annuli_exp_tokens[ 2 ] );

  calculate_wedges( &(annuli->wedge), annuli_exp_tokens[ 3 ],
		    annuli->num_wedges );

  /* print_AnnuliExp( stderr, annuli ); */

  str_tokenize_free( annuli_exp_tokens, 1 );
  tf_leave( );

  return annuli;

}

